unit RunUOControllerUnit;

interface

uses
  Classes,
  Windows;

type
  TStringEvent=procedure(sender: TObject; text: String) of object;

  TRunUOPipeOutputThread=class;

  TRunUOController=class
  private
    FAppName: String;
    FDirectory: String;
    FOnStandardError: TStringEvent;
    FOnStandardOutput: TStringEvent;
    FOnStart: TNotifyEvent;
    FOnStop: TNotifyEvent;
    FOutputThread: TRunUOPipeOutputThread;
    FRunUOProcess: TProcessInformation;
    FRunUOStartup: TStartupInfo;
    FRunning: Boolean;
    FStandardError: THandle;
    FStandardInput: THandle;
    FStandardOutput: THandle;
    function GetRunning: Boolean;
  public
    constructor Create(const appname, curdir: String);
    destructor Destroy; override;
    procedure SendString(const s: String);
    procedure Start;
    procedure Stop;
    property AppName: String read FAppName;
    property OnStandardError: TStringEvent read FOnStandardError
      write FOnStandardError;
    property OnStandardOutput: TStringEvent read FOnStandardOutput
      write FOnStandardOutput;
    property OnStart: TNotifyEvent read FOnStart write FOnStart;
    property OnStop: TNotifyEvent read FOnStop write FOnStop;
    property Running: Boolean read GetRunning;
  end;

  TRunUOPipeOutputThread=class(TThread)
  private
    FController: TRunUOController;
    FStdErr: THandle;
    FStdOut: THandle;
    FText: String;
    procedure DoError;
    procedure DoOutput;
  public
    constructor Create(const controller: TRunUOController;
      stdout, stderr: THandle);
    destructor Destroy; override;
    procedure Execute; override;
  end;

  TRunUOProcessThread=class(TThread)
  private
    FController: TRunUOController;
    FProcessHandle: THandle;
    FThreadHandle: THandle;
  public
    constructor Create(const controller: TRunUOController;
      proc, thread: THandle);
    destructor Destroy; override;
    procedure Execute; override;
  end;

implementation

//uses SysUtils;

{ TRunUOController }

constructor TRunUOController.Create(const appname, curdir: String);
begin
  FAppName:=appname;
  FDirectory:=curdir;
  FRunning:=False;
  with FRunUOStartup do
  begin
    cb:=sizeof(FRunUOStartup);
    lpReserved:=nil;
    lpDesktop:=nil;
    lpTitle:=nil;
    dwFlags:=STARTF_USESHOWWINDOW or STARTF_USESTDHANDLES;
    wShowWindow:=SW_HIDE;
    cbReserved2:=0;
    lpReserved2:=nil;
  end;
end;

procedure TRunUOController.Start;
var
  HInput, HOutput: THandle;
  sa: TSecurityAttributes;
begin
  sa.nLength:=sizeof(sa);
  sa.lpSecurityDescriptor:=nil;
  sa.bInheritHandle:=True;
  // Standard Input
  CreatePipe(HOutput, HInput, @sa, 0);
  FStandardInput:=HInput;
  FRunUOStartup.hStdInput:=HOutput;
  // Standard Output
  CreatePipe(HOutput, HInput, @sa, 0);
  FStandardOutput:=HOutput;
  FRunUOStartup.hStdOutput:=HInput;
  // Standard Error
  CreatePipe(HOutput, HInput, @sa, 0);
  FStandardError:=HOutput;
  FRunUOStartup.hStdError:=HInput;
  // Start output thread
  FOutputThread:=TRunUOPipeOutputThread.Create(self, FStandardOutput,
    FStandardError);
  FOutputThread.Resume;
  // Start process
  CreateProcess(@FAppName[1], @FAppName[1], nil, nil, True,
    NORMAL_PRIORITY_CLASS, nil, @FDirectory[1], FRunUOStartup, FRunUOProcess);
  FRunning:=True;
end;

destructor TRunUOController.Destroy;
begin
  Stop;
  inherited;
end;

procedure TRunUOController.SendString(const s: String);
var
  BytesWritten: DWord;
begin
  WriteFile(FStandardInput, s[1], Length(s), BytesWritten, nil);
end;

procedure TRunUOController.Stop;
begin
  if Running then
  begin
    TerminateThread(FRunUOProcess.hThread, 0);
    TerminateProcess(FRunUOProcess.hProcess, 0);
    CloseHandle(FRunUOProcess.hThread);
    CloseHandle(FRunUOProcess.hProcess);
    FOutputThread.Terminate;
    CloseHandle(FRunUOStartup.hStdError);
    CloseHandle(FStandardInput);
    CloseHandle(FRunUOStartup.hStdInput);
    CloseHandle(FRunUOStartup.hStdOutput);
//    FStandardErrorThread.Free;
//    FStandardOutputThread.Free;
  end;
  FRunning:=False;
end;

function TRunUOController.GetRunning: Boolean;
begin
  Result:=FRunning;
end;

{ TRunUOPipeOutputThread }

constructor TRunUOPipeOutputThread.Create(const controller: TRunUOController;
  stdout, stderr: THandle);
begin
  inherited Create(True);
  FController:=controller;
  FStdOut:=stdout;
  FStdErr:=stderr;
  FreeOnTerminate:=True;
end;

destructor TRunUOPipeOutputThread.Destroy;
begin
  inherited;
  CloseHandle(FStdOut);
  CloseHandle(FStdErr);
end;

procedure TRunUOPipeOutputThread.DoError;
begin
  if Assigned(FController) and Assigned(FController.FOnStandardError) then
    FController.FOnStandardError(FController, FText);
end;

procedure TRunUOPipeOutputThread.DoOutput;
begin
  if Assigned(FController) and Assigned(FController.FOnStandardOutput) then
    FController.FOnStandardOutput(FController, FText);
end;

procedure TRunUOPipeOutputThread.Execute;
const
  bufsize=8192;
var
  Buffer: array[0..bufsize-1] of Byte;
  BytesRead: DWord;
begin
  while not Terminated do
  begin
    if PeekNamedPipe(FStdOut, @Buffer[0], 1, @BytesRead, nil, nil) then
    begin
      if BytesRead>0 then
      begin
        SetLength(FText, bufsize);
        ReadFile(FStdOut, Buffer[0], 8192, BytesRead, nil);
        if (BytesRead>0) and Assigned(FController.FOnStandardOutput) then
        begin
          SetLength(FText, BytesRead);
          Move(Buffer[0], FText[1], BytesRead);
          Synchronize(DoOutput);
        end;
      end
      else if PeekNamedPipe(FStdErr, @Buffer[0], 1, @BytesRead, nil, nil) then
      begin
        if BytesRead>0 then
        begin
          SetLength(FText, bufsize);
          ReadFile(FStdErr, Buffer[0], 8192, BytesRead, nil);
          if (BytesRead>0) and Assigned(FController.FOnStandardError) then
          begin
            SetLength(FText, BytesRead);
            Move(Buffer[0], FText[1], BytesRead);
            Synchronize(DoError);
          end;
        end
        else
        begin
          Sleep(200);
        end;
      end;
    end;
  end;
end;

{ TRunUOProcessThread }

constructor TRunUOProcessThread.Create(const controller: TRunUOController;
  proc, thread: THandle);
begin
  inherited Create(false);
  FController:=controller;
  FProcessHandle:=proc;
  FThreadHandle:=thread;
end;

destructor TRunUOProcessThread.Destroy;
begin

  inherited;
end;

procedure TRunUOProcessThread.Execute;
begin
  inherited;

end;

end.
